import { fileURLToPath } from "node:url";
import { copyFileSync, existsSync, mkdirSync, readFile, readdirSync, rmSync, statSync, writeFile } from "node:fs";
import detectIndent from "detect-indent";
import { detectNewline } from "detect-newline";
import { componentsPath, dirname, functionsPath, join, projectPath } from "@stacksjs/paths";
import { contains } from "@stacksjs/arrays";
export async function readJsonFile(name, cwd) {
  const file = await readTextFile(name, cwd);
  const data = JSON.parse(file.data);
  const indent = detectIndent(file.data).indent;
  const newline = detectNewline(file.data);
  return { ...file, data, indent, newline };
}
export async function writeJsonFile(file) {
  let json = JSON.stringify(file.data, void 0, file.indent);
  if (file.newline)
    json += file.newline;
  return writeTextFile({ ...file, data: json });
}
export function readTextFile(name, cwd) {
  return new Promise((resolve, reject) => {
    let filePath;
    if (cwd)
      filePath = join(cwd, name);
    else
      filePath = name;
    readFile(filePath, "utf8", (err, text) => {
      if (err) {
        reject(err);
      } else {
        resolve({
          path: filePath,
          data: text
        });
      }
    });
  });
}
export function writeTextFile(file) {
  return new Promise((resolve, reject) => {
    writeFile(file.path, file.data, (err) => {
      if (err)
        reject(err);
      else
        resolve();
    });
  });
}
export function isFolder(path) {
  try {
    return statSync(path).isDirectory();
  } catch {
    return false;
  }
}
export function isFile(path) {
  return existsSync(projectPath(path));
}
export function hasFiles(folder) {
  return readdirSync(folder).length > 0;
}
export function hasComponents() {
  return hasFiles(componentsPath());
}
export function hasFunctions() {
  return hasFiles(functionsPath());
}
export function copyFolder(src, dest, exclude = []) {
  if (!existsSync(dest))
    mkdirSync(dest, { recursive: true });
  if (existsSync(src)) {
    readdirSync(src).forEach((file) => {
      if (!contains(join(src, file), exclude)) {
        const srcPath = join(src, file);
        const destPath = join(dest, file);
        if (statSync(srcPath).isDirectory())
          copyFolder(srcPath, destPath, exclude);
        else
          copyFileSync(srcPath, destPath);
      }
    });
  }
}
export async function deleteFolder(path) {
  if (statSync(path).isDirectory())
    await rmSync(path, { recursive: true, force: true });
}
export function deleteFiles(dir, exclude = []) {
  if (existsSync(dir)) {
    readdirSync(dir).forEach((file) => {
      const path = join(dir, file);
      if (statSync(path).isDirectory()) {
        if (readdirSync(path).length === 0)
          rmSync(path, { recursive: true, force: true });
        else
          deleteFiles(path, exclude);
      } else if (!contains(path, exclude)) {
        rmSync(path);
      }
    });
  }
}
export function deleteEmptyFolders(dir) {
  if (existsSync(dir)) {
    readdirSync(dir).forEach((file) => {
      const path = join(dir, file);
      if (statSync(path).isDirectory()) {
        if (readdirSync(path).length === 0)
          rmSync(path, { recursive: true, force: true });
        else
          deleteEmptyFolders(path);
      }
    });
  }
}
export function doesFolderExist(path) {
  return existsSync(path);
}
export const _dirname = typeof __dirname !== "undefined" ? __dirname : dirname(fileURLToPath(import.meta.url));
